@@ -31,7 +31,7 @@
 # The fact that you are presently reading this means that you have had
 # knowledge of the CeCILL-C license and that you accept its terms.
 #
-# $Id: Clush.py 484 2011-03-10 20:08:59Z st-cea $
+# $Id: Clush.py 490 2011-03-19 16:09:05Z st-cea $
 
 """
 execute cluster commands in parallel
@@ -532,7 +532,7 @@ def run_command(task, cmd, ns, timeout, display):
 
     if (display.gather or display.line_mode) and ns is not None:
         if display.gather and display.line_mode:
-            handler = LiveGatherOutputHandler(display)
+            handler = LiveGatherOutputHandler(display, ns)
         else:
             handler = GatherOutputHandler(display)
 
