@@ -1,5 +1,5 @@
 #
-# Copyright CEA/DAM/DIF (2009, 2010)
+# Copyright CEA/DAM/DIF (2009, 2010, 2011)
 #  Contributor: Stephane THIELL <stephane.thiell@cea.fr>
 #
 # This file is part of the ClusterShell library.
@@ -30,7 +30,7 @@
 # The fact that you are presently reading this means that you have had
 # knowledge of the CeCILL-C license and that you accept its terms.
 #
-# $Id: EPoll.py 384 2010-10-17 21:24:21Z st-cea $
+# $Id: EPoll.py 460 2011-02-07 23:22:45Z st-cea $
 
 """
 A ClusterShell Engine using epoll, an I/O event notification facility.
@@ -67,7 +67,7 @@ class EngineEPoll(Engine):
             # get an epoll object
             self.epolling = select.epoll()
         except AttributeError:
-            raise EngineNotSupportedError()
+            raise EngineNotSupportedError(EngineEPoll.identifier)
 
     def _register_specific(self, fd, event):
         """
