@@ -1,5 +1,5 @@
 #
-# Copyright CEA/DAM/DIF (2008, 2009, 2010)
+# Copyright CEA/DAM/DIF (2008, 2009, 2010, 2011)
 #  Contributor: Stephane THIELL <stephane.thiell@cea.fr>
 #
 # This file is part of the ClusterShell library.
@@ -30,7 +30,7 @@
 # The fact that you are presently reading this means that you have had
 # knowledge of the CeCILL-C license and that you accept its terms.
 #
-# $Id: Ssh.py 418 2010-11-30 21:42:42Z st-cea $
+# $Id: Ssh.py 459 2011-02-07 23:13:55Z st-cea $
 
 """
 ClusterShell Ssh/Scp support
@@ -86,7 +86,7 @@ class Ssh(EngineClient):
         # Add custom ssh options
         ssh_options = task.info("ssh_options")
         if ssh_options:
-            cmd_l.append(ssh_options)
+            cmd_l += ssh_options.split()
 
         cmd_l.append("%s" % self.key)
         cmd_l.append("%s" % self.command)
@@ -140,29 +140,35 @@ class Ssh(EngineClient):
         Handle a read notification. Called by the engine as the result of an
         event indicating that a read is available.
         """
-        debug = self.worker.task.info("debug", False)
+        # Local variables optimization
+        worker = self.worker
+        task = worker.task
+        key = self.key
+        node_msgline = worker._on_node_msgline
+        debug = task.info("debug", False)
         if debug:
-            print_debug = self.worker.task.info("print_debug")
-
+            print_debug = task.info("print_debug")
         for msg in self._readlines():
             if debug:
-                print_debug(self.worker.task, "%s: %s" % (self.key, msg))
-            # handle full msg line
-            self.worker._on_node_msgline(self.key, msg)
+                print_debug(task, "%s: %s" % (key, msg))
+            node_msgline(key, msg)  # handle full msg line
 
     def _handle_error(self):
         """
         Handle a read error (stderr) notification.
         """
-        debug = self.worker.task.info("debug", False)
+        # Local variables optimization
+        worker = self.worker
+        task = worker.task
+        key = self.key
+        node_errline = worker._on_node_errline
+        debug = task.info("debug", False)
         if debug:
-            print_debug = self.worker.task.info("print_debug")
-
+            print_debug = task.info("print_debug")
         for msg in self._readerrlines():
             if debug:
-                print_debug(self.worker.task, "%s@STDERR: %s" % (self.key, msg))
-            # handle full msg line
-            self.worker._on_node_errline(self.key, msg)
+                print_debug(task, "%s@STDERR: %s" % (key, msg))
+            node_errline(key, msg)  # handle full stderr line
 
 
 class Scp(Ssh):
@@ -228,7 +234,7 @@ class Scp(Ssh):
         for key in [ "ssh_options", "scp_options" ]:
             ssh_options = task.info(key)
             if ssh_options:
-                cmd_l.append(ssh_options)
+                cmd_l += ssh_options.split()
 
         if self.reverse:
             user = task.info("ssh_user")
@@ -327,9 +333,11 @@ class WorkerSsh(DistantWorker):
 
     def _check_fini(self):
         if self._close_count >= len(self.clients):
-            if self._has_timeout:
-                self._invoke("ev_timeout")
-            self._invoke("ev_close")
+            handler = self.eh
+            if handler:
+                if self._has_timeout:
+                    handler.ev_timeout(self)
+                handler.ev_close(self)
 
     def write(self, buf):
         """
