@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # ClusterShell.NodeSet.RangeSet test suite
 # Written by S. Thiell
-# $Id: RangeSetTest.py 422 2010-12-08 23:00:18Z st-cea $
+# $Id: RangeSetTest.py 485 2011-03-12 10:09:26Z st-cea $
 
 
 """Unit test for RangeSet"""
@@ -23,7 +23,7 @@ class RangeSetTest(unittest.TestCase):
         self.assertEqual(len(r1), length)
 
     def testSimple(self):
-        """test simple ranges"""
+        """test RangeSet simple ranges"""
         self._testRS("0", "0", 1)
         self._testRS("1", "1", 1)
         self._testRS("0-2", "0-2", 3)
@@ -32,14 +32,14 @@ class RangeSetTest(unittest.TestCase):
         self._testRS("1-3,4-6,7-10", "1-10", 10)
 
     def testStepSimple(self):
-        """test simple step usages"""
+        """test RangeSet simple step usages"""
         self._testRS("0-4/2", "0-4/2", 3)
         self._testRS("1-4/2", "1,3", 2)
         self._testRS("1-4/3", "1,4", 2)
         self._testRS("1-4/4", "1", 1)
 
     def testStepAdvanced(self):
-        """test advanced step usages"""
+        """test RangeSet advanced step usages"""
         self._testRS("1-4/4,2-6/2", "1-2,4,6", 4)
         self._testRS("6-24/6,9-21/6", "6-24/3", 7)
         self._testRS("0-24/2,9-21/2", "0-8/2,9-22,24", 20)
@@ -60,7 +60,7 @@ class RangeSetTest(unittest.TestCase):
         self._testRS("1-17/2,33-41/2,2-19/2", "1-18,33-41/2", 23)
 
     def testIntersectSimple(self):
-        """test simple intersections of ranges"""
+        """test RangeSet with simple intersections of ranges"""
         r1 = RangeSet("4-34")
         r2 = RangeSet("27-42")
         r1.intersection_update(r2)
@@ -85,7 +85,7 @@ class RangeSetTest(unittest.TestCase):
 
 
     def testIntersectStep(self):
-        """test more intersections of ranges"""
+        """test RangeSet with more intersections of ranges"""
         r1 = RangeSet("4-34/2")
         r2 = RangeSet("28-42/2")
         r1.intersection_update(r2)
@@ -105,7 +105,7 @@ class RangeSetTest(unittest.TestCase):
         self.assertEqual(len(r1), 8)
 
     def testSubSimple(self):
-        """test simple difference of ranges"""
+        """test RangeSet with simple difference of ranges"""
         r1 = RangeSet("4,7-33")
         r2 = RangeSet("8-33")
         r1.difference_update(r2)
@@ -122,7 +122,7 @@ class RangeSetTest(unittest.TestCase):
         self.assertEqual(len(r3), 2)
 
     def testSymmetricDifference(self):
-        """test symmetric difference of ranges"""
+        """test RangeSet.symmetric_difference_update()"""
         r1 = RangeSet("4,7-33")
         r2 = RangeSet("8-34")
         r1.symmetric_difference_update(r2)
@@ -140,7 +140,7 @@ class RangeSetTest(unittest.TestCase):
 
 
     def testSubStep(self):
-        """test more sub of ranges (with step)"""
+        """test RangeSet with more sub of ranges (with step)"""
         # case 1 no sub
         r1 = RangeSet("4-34/2", autostep=3)
         r2 = RangeSet("3-33/2", autostep=3)
@@ -213,7 +213,7 @@ class RangeSetTest(unittest.TestCase):
         self.assertEqual(len(r1), 31)
 
     def testContains(self):
-        """test RangeSet __contains__()"""
+        """test RangeSet.__contains__()"""
         r1 = RangeSet("1-100,102,105-242,800")
         self.assertEqual(len(r1), 240)
         self.assert_(99 in r1)
@@ -245,7 +245,7 @@ class RangeSetTest(unittest.TestCase):
         self.assert_(not "03" in r4)
 
     def testIsSuperSet(self):
-        """test RangeSet issuperset()"""
+        """test RangeSet.issuperset()"""
         r1 = RangeSet("1-100,102,105-242,800")
         self.assertEqual(len(r1), 240)
         r2 = RangeSet("3-98,140-199,800")
@@ -261,7 +261,7 @@ class RangeSetTest(unittest.TestCase):
         self.assert_(not r1 > r2)
 
     def testIsSubSet(self):
-        """test RangeSet issubset()"""
+        """test RangeSet.issubset()"""
         r1 = RangeSet("1-100,102,105-242,800-900/2")
         r2 = RangeSet("3,800,802,804,888")
         self.assert_(r2.issubset(r2))
@@ -272,7 +272,7 @@ class RangeSetTest(unittest.TestCase):
         self.assert_(not r1 <= r2)
 
     def testGetItem(self):
-        """test RangeSet __getitem__()"""
+        """test RangeSet.__getitem__()"""
         r1 = RangeSet("1-100,102,105-242,800")
         self.assertEqual(len(r1), 240)
         self.assertEqual(r1[0], 1)
@@ -286,6 +286,14 @@ class RangeSetTest(unittest.TestCase):
         self.assertEqual(r1[237], 241)
         self.assertEqual(r1[238], 242)
         self.assertEqual(r1[239], 800)
+        # negative indices
+        self.assertEqual(r1[-1], 800)
+        self.assertEqual(r1[-240], 1)
+        for n in range(1, len(r1)):
+            self.assertEqual(r1[-n], r1[len(r1)-n])
+        self.assertRaises(IndexError, r1.__getitem__, -len(r1)-1)
+        self.assertRaises(IndexError, r1.__getitem__, -len(r1)-2)
+
         r2 = RangeSet("1-37/3,43-52/3,58-67/3,73-100/3,102-106/2")
         self.assertEqual(len(r2), 34)
         self.assertEqual(r2[0], 1)
@@ -304,10 +312,40 @@ class RangeSetTest(unittest.TestCase):
         #self.assertRaises(TypeError, r2['foo'])
 
     def testGetSlice(self):
-        """test RangeSet __getslice__()"""
+        """test RangeSet.__getitem__() with slice"""
         r0 = RangeSet("1-12")
         self.assertEqual(r0[0:3], RangeSet("1-3"))
         self.assertEqual(r0[2:7], RangeSet("3-7"))
+        # negative indices - sl_start
+        self.assertEqual(r0[-1:0], RangeSet())
+        self.assertEqual(r0[-2:0], RangeSet())
+        self.assertEqual(r0[-11:0], RangeSet())
+        self.assertEqual(r0[-12:0], RangeSet())
+        self.assertEqual(r0[-13:0], RangeSet())
+        self.assertEqual(r0[-1000:0], RangeSet())
+        self.assertEqual(r0[-1:], RangeSet("12"))
+        self.assertEqual(r0[-2:], RangeSet("11-12"))
+        self.assertEqual(r0[-11:], RangeSet("2-12"))
+        self.assertEqual(r0[-12:], RangeSet("1-12"))
+        self.assertEqual(r0[-13:], RangeSet("1-12"))
+        self.assertEqual(r0[-1000:], RangeSet("1-12"))
+        self.assertEqual(r0[-13:1], RangeSet("1"))
+        self.assertEqual(r0[-13:2], RangeSet("1-2"))
+        self.assertEqual(r0[-13:11], RangeSet("1-11"))
+        self.assertEqual(r0[-13:12], RangeSet("1-12"))
+        self.assertEqual(r0[-13:13], RangeSet("1-12"))
+        # negative indices - sl_stop
+        self.assertEqual(r0[0:-1], RangeSet("1-11"))
+        self.assertEqual(r0[:-1], RangeSet("1-11"))
+        self.assertEqual(r0[0:-2], RangeSet("1-10"))
+        self.assertEqual(r0[:-2], RangeSet("1-10"))
+        self.assertEqual(r0[1:-2], RangeSet("2-10"))
+        self.assertEqual(r0[4:-4], RangeSet("5-8"))
+        self.assertEqual(r0[5:-5], RangeSet("6-7"))
+        self.assertEqual(r0[6:-5], RangeSet("7"))
+        self.assertEqual(r0[6:-6], RangeSet())
+        self.assertEqual(r0[7:-6], RangeSet())
+        self.assertEqual(r0[0:-1000], RangeSet())
 
         r1 = RangeSet("10-14,16-20")
         self.assertEqual(r1[2:6], RangeSet("12-14,16"))
@@ -349,6 +387,19 @@ class RangeSetTest(unittest.TestCase):
         self.assertEqual(r1[1:6:2], RangeSet("2,9,11"))
         self.assertEqual(r1[1:7:2], RangeSet("2,9,11"))
 
+        # negative indices - sl_step
+        self.assertRaises(IndexError, r1.__getitem__, slice(1, None, -2))
+        self.assertEqual(r1[::-2], RangeSet("1,4,10,12"))
+        r2 = RangeSet("1-2,4,9,10-13")
+        self.assertEqual(r2[::-2], RangeSet("2,9,11,13"))
+        self.assertEqual(r2[::-3], RangeSet("2,10,13"))
+        self.assertEqual(r2[::-4], RangeSet("9,13"))
+        self.assertEqual(r2[::-5], RangeSet("4,13"))
+        self.assertEqual(r2[::-6], RangeSet("2,13"))
+        self.assertEqual(r2[::-7], RangeSet("1,13"))
+        self.assertEqual(r2[::-8], RangeSet("13"))
+        self.assertEqual(r2[::-9], RangeSet("13"))
+
         # Partial slices
         self.assertEqual(r1[2:], RangeSet("4,9-12"))
         self.assertEqual(r1[:3], RangeSet("1-2,4"))
@@ -372,7 +423,7 @@ class RangeSetTest(unittest.TestCase):
         self.assertEqual(r3[300:3890:2], RangeSet("300-3889/2", autostep=2))
 
     def testSplit(self):
-        """test RangeSet split()"""
+        """test RangeSet.split()"""
         # Empty rangeset
         rangeset = RangeSet()
         self.assertEqual(len(list(rangeset.split(2))), 0)
@@ -391,7 +442,7 @@ class RangeSetTest(unittest.TestCase):
                              tuple(rangeset.split(i)))
 
     def testAdd(self):
-        """test RangeSet add()"""
+        """test RangeSet.add()"""
         r1 = RangeSet("1-100,102,105-242,800")
         self.assertEqual(len(r1), 240)
         r1.add(801)
@@ -406,7 +457,7 @@ class RangeSetTest(unittest.TestCase):
         self.assertEqual(len(r1), 243)
 
     def testUnion(self):
-        """test RangeSet union()"""
+        """test RangeSet.union()"""
         r1 = RangeSet("1-100,102,105-242,800")
         self.assertEqual(len(r1), 240)
         r2 = RangeSet("243-799,1924-1984")
@@ -427,16 +478,19 @@ class RangeSetTest(unittest.TestCase):
         self.assertEqual(str(r4), "1-100,102,105-800")
 
     def testRemove(self):
-        """test RangeSet remove()"""
+        """test RangeSet.remove()"""
         r1 = RangeSet("1-100,102,105-242,800")
         self.assertEqual(len(r1), 240)
         r1.remove(100)
         self.assertEqual(len(r1), 239)
         self.assertEqual(str(r1), "1-99,102,105-242,800")
         self.assertRaises(KeyError, r1.remove, 101)
+        # test remove integer-castable type (convenience)
+        r1.remove("106")
+        self.assertRaises(KeyError, r1.remove, "foo")
 
     def testClear(self):
-        """test RangeSet clear()"""
+        """test RangeSet.clear()"""
         r1 = RangeSet("1-100,102,105-242,800")
         self.assertEqual(len(r1), 240)
         self.assertEqual(str(r1), "1-100,102,105-242,800")
@@ -445,7 +499,7 @@ class RangeSetTest(unittest.TestCase):
         self.assertEqual(str(r1), "")
     
     def testFromListConstructor(self):
-        """test RangeSet.fromlist constructor"""
+        """test RangeSet.fromlist() constructor"""
         rgs = RangeSet.fromlist([ "3", "5-8", "1" ])
         self.assertEqual(str(rgs), "1,3,5-8")
         self.assertEqual(len(rgs), 6)
@@ -508,13 +562,13 @@ class RangeSetTest(unittest.TestCase):
         self.assert_(good_error, "TypeError not raised for ^")
 
     def testIsSubSetError(self):
-        """test RangeSet issubset error"""
+        """test RangeSet.issubset() error"""
         rg1 = RangeSet("1-5")
         rg2 = "4-6"
         self.assertRaises(TypeError, rg1.issubset, rg2)
 
     def testEquality(self):
-        """test RangeSet equal"""
+        """test RangeSet equality"""
         rg0_1 = RangeSet()
         rg0_2 = RangeSet()
         self.assertEqual(rg0_1, rg0_2)
@@ -534,6 +588,108 @@ class RangeSetTest(unittest.TestCase):
         else:
             self.fail("rg1 != None failed")
 
+    def testAddRange(self):
+        """test RangeSet.add_range()"""
+        r1 = RangeSet()
+        r1.add_range(1, 100, 1)
+        self.assertEqual(len(r1), 99)
+        self.assertEqual(str(r1), "1-99")
+        r1.add_range(40, 101, 1)
+        self.assertEqual(len(r1), 100)
+        self.assertEqual(str(r1), "1-100")
+        r1.add_range(399, 423, 2)
+        self.assertEqual(len(r1), 112)
+        self.assertEqual(str(r1), "1-100,399,401,403,405,407,409,411,413,415,417,419,421")
+        # With autostep...
+        r1 = RangeSet(autostep=2)
+        r1.add_range(1, 100, 1)
+        self.assertEqual(len(r1), 99)
+        self.assertEqual(str(r1), "1-99")
+        r1.add_range(40, 101, 1)
+        self.assertEqual(len(r1), 100)
+        self.assertEqual(str(r1), "1-100")
+        r1.add_range(399, 423, 2)
+        self.assertEqual(len(r1), 112)
+        self.assertEqual(str(r1), "1-100,399-421/2")
+        # Bound checks
+        r1 = RangeSet("1-30", autostep=2)
+        self.assertEqual(len(r1), 30)
+        self.assertEqual(str(r1), "1-30")
+        r1.add_range(32, 35, 1)
+        self.assertEqual(len(r1), 33)
+        self.assertEqual(str(r1), "1-30,32-34")
+        r1.add_range(31, 32, 1)
+        self.assertEqual(len(r1), 34)
+        self.assertEqual(str(r1), "1-34")
+        r1 = RangeSet("1-30/4")
+        self.assertEqual(len(r1), 8)
+        self.assertEqual(str(r1), "1,5,9,13,17,21,25,29")
+        r1.add_range(30, 32, 1)
+        self.assertEqual(len(r1), 10)
+        self.assertEqual(str(r1), "1,5,9,13,17,21,25,29-31")
+        r1.add_range(40, 65, 10)
+        self.assertEqual(len(r1), 13)
+        self.assertEqual(str(r1), "1,5,9,13,17,21,25,29-31,40,50,60")
+        r1 = RangeSet("1-30", autostep=2)
+        r1.add_range(40, 65, 10)
+        self.assertEqual(len(r1), 33)
+        self.assertEqual(str(r1), "1-30,40-60/10")
+        # One
+        r1.add_range(103, 104)
+        self.assertEqual(len(r1), 34)
+        self.assertEqual(str(r1), "1-30,40-60/10,103")
+        # Zero
+        self.assertRaises(AssertionError, r1.add_range, 103, 103)
+
+    def testSlices(self):
+        """test RangeSet.slices()"""
+        r1 = RangeSet()
+        self.assertEqual(len(r1), 0)
+        self.assertEqual(len(list(r1.slices())), 0)
+        # With padding, without autostep
+        r1 = RangeSet("1-7/2,8-12,3000-3019")
+        self.assertEqual(len(r1), 29)
+        self.assertEqual(list(r1.slices()), [(slice(1, 2, 1), 0), \
+            (slice(3, 4, 1), 0), (slice(5, 6, 1), 0), (slice(7, 13, 1), 0), \
+            (slice(3000, 3020, 1), 0)])
+        # With padding, with autostep
+        r1 = RangeSet("1-7/2,8-12,3000-3019", autostep=2)
+        self.assertEqual(len(r1), 29)
+        self.assertEqual(list(r1.slices()), [(slice(1, 8, 2), 0), \
+            (slice(8, 13, 1), 0), (slice(3000, 3020, 1), 0)])
+        # Without padding, without autostep
+        r1 = RangeSet("1-7/2,8-12,3000-3019")
+        self.assertEqual(len(r1), 29)
+        self.assertEqual(list(r1.slices(False)), [slice(1, 2, 1), \
+            slice(3, 4, 1), slice(5, 6, 1), slice(7, 13, 1), \
+            slice(3000, 3020, 1)])
+        # Without padding, with autostep
+        r1 = RangeSet("1-7/2,8-12,3000-3019", autostep=2)
+        self.assertEqual(len(r1), 29)
+        self.assertEqual(list(r1.slices(False)), [slice(1, 8, 2), \
+            slice(8, 13, 1), slice(3000, 3020, 1)])
+
+    def testCopy(self):
+        """test RangeSet.copy()"""
+        rangeset = RangeSet("115-117,130,166-170,4780-4999")
+        self.assertEqual(len(rangeset), 229)
+        self.assertEqual(str(rangeset), "115-117,130,166-170,4780-4999")
+        r1 = rangeset.copy()
+        r2 = rangeset.copy()
+        self.assertEqual(rangeset, r1) # content equality
+        r1.remove(166)
+        self.assertEqual(len(rangeset), len(r1) + 1)
+        self.assertNotEqual(rangeset, r1)
+        self.assertEqual(str(rangeset), "115-117,130,166-170,4780-4999")
+        self.assertEqual(str(r1), "115-117,130,167-170,4780-4999")
+        r2.update(RangeSet("118"))
+        self.assertNotEqual(rangeset, r2)
+        self.assertNotEqual(r1, r2)
+        self.assertEqual(len(rangeset) + 1, len(r2))
+        self.assertEqual(str(rangeset), "115-117,130,166-170,4780-4999")
+        self.assertEqual(str(r1), "115-117,130,167-170,4780-4999")
+        self.assertEqual(str(r2), "115-118,130,166-170,4780-4999")
+
 
 if __name__ == '__main__':
     suite = unittest.TestLoader().loadTestsFromTestCase(RangeSetTest)
